@@ -21,7 +21,7 @@
    PURPOSE. */
 
 
-/* $Header: /Software/SPIM/src/buttons.c 5     12/24/00 1:37p Larus $
+/* $Header: /Software/SPIM/src/buttons.c 7     2/01/01 8:57p Larus $
  */
 
 #include <X11/Intrinsic.h>
@@ -718,6 +718,7 @@ reload_action (w, client_data, call_data)
 
 #define CLEAR_REGS 1
 #define CLEAR_MEM_REGS 2
+#define CLEAR_CONSOLE 3
 
 #ifdef __STDC__
 static void
@@ -746,6 +747,11 @@ add_clear_button (parent)
 				 args, ONE);
   XtAddCallback (entry, XtNcallback, clear_program_state_action,
 		 (XtPointer) CLEAR_MEM_REGS);
+
+  entry = XtCreateManagedWidget ("console display", smeBSBObjectClass, menu,
+				 args, ONE);
+  XtAddCallback (entry, XtNcallback, clear_program_state_action,
+		 (XtPointer) CLEAR_CONSOLE);
 }
 
 
@@ -776,6 +782,11 @@ clear_program_state_action (w, client_data, call_data)
       stack_initialized = 0;
       break;
 
+    case CLEAR_CONSOLE:
+      write_output (message_out, "Console display cleared\n\n");
+      clear_console_display ();
+      break;
+
     default:
       fatal_error("Unknown action: %d\n", clear_op);
     }
@@ -854,7 +865,17 @@ set_value_action (location_str, value_str)
   unsigned long value;
   int reg_no;
 
-  value = strtol (value_str, NULL, 0);
+  if (strlen (value_str) == 3
+      && value_str[0] == '\''
+      && value_str[2] == '\'')
+    {
+      /* Literal of the form: 'c' */
+      value = value_str[1];
+    }
+  else
+    {
+      value = strtol (value_str, NULL, 0);
+    }
   reg_no = register_name_to_number (location_str);
 
   if (reg_no < 0)
