@@ -24,7 +24,7 @@
    PURPOSE. */
 
 
-/* $Header: /Software/SPIM/src/mips-syscall.c 7     12/24/00 1:37p Larus $ */
+/* $Header: /Software/SPIM/src/mips-syscall.c 11    1/12/02 4:16p Larus $ */
 
 
 #include <stdio.h>
@@ -51,6 +51,9 @@
 #include <sys/syscall.h>
 #endif
 
+#ifdef WIN32
+#include <io.h>
+#endif
 
 #include "spim.h"
 #include "inst.h"
@@ -60,11 +63,10 @@
 #include "spim-syscall.h"
 #include "mips-syscall.h"
 
-
+#ifndef WIN32
 /* Imported variables: */
-
 extern int errno;
-
+#endif
 
 /* Imported functions: */
 
@@ -472,9 +474,68 @@ do_syscall ()
 	    break;
 	  }
 
+	case PRINT_CHARACTER_SYSCALL:
+	  write_output (console_out, "%c", R[REG_A0]);
+	  break;
+
+	case READ_CHARACTER_SYSCALL:
+	  {
+	    static char str [2];
+
+	    read_input (str, 2);
+	    if (*str == '\0') *str = '\n';      /* makes xspim = spim */
+	    R[REG_RES] = (long) str[0];
+	    break;
+	  }
+
 	case EXIT_SYSCALL:
 	  return (0);
 
+	case OPEN_SYSCALL:
+	  {
+#ifdef WIN32
+	    R[REG_RES] = _open(MEM_ADDRESS(R[REG_A0]), R[REG_A1], R[REG_A2]);
+#else
+	    R[REG_RES] = open(MEM_ADDRESS(R[REG_A0]), R[REG_A1], R[REG_A2]);
+#endif
+	    break;
+	  }
+		
+	case READ_SYSCALL:
+	  {
+	    /* Test if address is valid */
+	    MEM_ADDRESS(R[REG_A1] + R[REG_A2] - 1);
+#ifdef WIN32
+	    R[REG_RES] = _read(R[REG_A0], MEM_ADDRESS(R[REG_A1]), R[REG_A2]);
+#else
+	    R[REG_RES] = read(R[REG_A0], MEM_ADDRESS(R[REG_A1]), R[REG_A2]);
+#endif
+	    data_modified = 1;
+	    break;
+	  }
+
+	case WRITE_SYSCALL:
+	  {
+	    /* Test if address is valid */
+	    MEM_ADDRESS(R[REG_A1] + R[REG_A2] - 1);
+#ifdef WIN32
+	    R[REG_RES] = _write(R[REG_A0], MEM_ADDRESS(R[REG_A1]), R[REG_A2]);
+#else
+	    R[REG_RES] = write(R[REG_A0], MEM_ADDRESS(R[REG_A1]), R[REG_A2]);
+#endif
+	    break;
+	  }
+
+	case CLOSE_SYSCALL:
+	  {
+#ifdef WIN32
+	    R[REG_RES] = _close(R[REG_A0]);
+#else
+	    R[REG_RES] = close(R[REG_A0]);
+#endif
+	    break;
+	  }
+
 	default:
 	  run_error ("Unknown system call: %d\n", R[REG_V0]);
 	  break;
@@ -754,7 +815,7 @@ do_syscall ()
 
 
 /* Execute a Unix system call.  Returns negative on error. */
-
+#if (mips && dec)
 #ifdef __STDC__
 static int
 unixsyscall (void)
@@ -763,7 +824,6 @@ static int
 unixsyscall ()
 #endif
 {
-#if (mips && dec)
   void* arg0, arg1, arg2, arg3;
 
   arg0 = SYSCALL_ARG (REG_V0,arg0, REG_A0);
@@ -788,12 +848,11 @@ unixsyscall ()
       R[REG_ERR] = 0;
       return (R[REG_RES]);
     }
-#else
-  return 0;
-#endif
 }
+#endif
 
 
+#if (mips && dec)
 #ifdef __STDC__
 static int
 reverse_fds (int fd)
@@ -812,6 +871,7 @@ reverse_fds (fd)
   run_error ("Couldn't reverse translate fds\n");
   return (-1);
 }
+#endif
 
 
 #ifdef __STDC__
@@ -835,17 +895,17 @@ print_syscall_usage ()
 
 
 #ifdef __STDC__
-void 
+void
 initialize_prog_fds (void)
 #else
-void 
+void
 initialize_prog_fds ()
 #endif
 {
   int x;
 
   for (x = 0; x < OPEN_MAX; prog_fds[x++] = -1);
-  if (((prog_fds[0] = dup(0)) < 0) || 
+  if (((prog_fds[0] = dup(0)) < 0) ||
       ((prog_fds[1] = dup(1)) < 0) ||
       ((prog_fds[2] = dup(2)) < 0))
     error("init_prog_fds");
@@ -855,10 +915,10 @@ initialize_prog_fds ()
 /* clear out programs file descriptors, close necessary files */
 
 #ifdef __STDC__
-void 
+void
 kill_prog_fds (void)
 #else
-void 
+void
 kill_prog_fds ()
 #endif
 {
@@ -967,7 +1027,7 @@ handle_exception ()
     }
 }
 
-
+#if (mips && dec)
 #ifdef __STDC__
 static void
 setup_signal_stack (void)
@@ -976,7 +1036,6 @@ static void
 setup_signal_stack ()
 #endif
 {
-#if (mips && dec)
   int i;
   struct sigcontext *sc;
 
@@ -997,10 +1056,10 @@ setup_signal_stack ()
   sc->sc_cause = Cause;		/* cp0 cause register */
   sc->sc_badvaddr = BadVAddr;	/* cp0 bad virtual address */
   sc->sc_badpaddr = 0;		/* cpu bd bad physical address */
-#endif
 }
+#endif
 
-
+#if (mips && dec)
 #ifdef __STDC__
 static void
 do_sigreturn (mem_addr sigptr)
@@ -1010,7 +1069,6 @@ do_sigreturn (sigptr)
   mem_addr sigptr;
 #endif
 {
-#if (mips && dec)
   int i;
   struct sigcontext *sc;
 
@@ -1026,5 +1084,5 @@ do_sigreturn (sigptr)
   Cause = sc->sc_cause;
   BadVAddr = sc->sc_badvaddr;
   R[29] += sizeof(struct sigcontext) + 4;
-#endif
 }
+#endif
